import os
from matplotlib import pyplot
import progressbar
import sys
import numpy as np
import tkinter as tk
from tkinter.filedialog import askdirectory
import tensorflow as tf
sys.path.append('C:\\Users\\{}\\Documents\\TensorFlow\\models\\research'.format(os.getlogin()))
from object_detection.utils import label_map_util

# Settings
MIN_SCORE=0.85
PATH_TO_CKPT="../../RCNN/frozen_inference_graph_v3.pb"
PATH_TO_LABEL="../../RCNN/label_map.pbtxt"



def load_model(path_to_ckpt, path_to_label, n_classes):

	label_map = label_map_util.load_labelmap(path_to_label)
	categories = label_map_util.convert_label_map_to_categories(label_map, max_num_classes=n_classes, use_display_name=True)
	category_index = label_map_util.create_category_index(categories)

	# Load the Tensorflow model into memory.
	detection_graph = tf.Graph()
	with detection_graph.as_default():
		od_graph_def = tf.compat.v1.GraphDef()
		with tf.io.gfile.GFile(path_to_ckpt, 'rb') as fid:
			serialized_graph = fid.read()
			od_graph_def.ParseFromString(serialized_graph)
			tf.import_graph_def(od_graph_def, name='')

		sess = tf.compat.v1.Session(graph=detection_graph)


	# Define input and output tensors (i.e. data) for the object detection classifier

	# Input tensor is the image
	image_tensor = detection_graph.get_tensor_by_name('image_tensor:0')

	# Output tensors are the detection boxes, scores, and classes
	# Each box represents a part of the image where a particular object was detected
	detection_boxes = detection_graph.get_tensor_by_name('detection_boxes:0')

	# Each score represents level of confidence for each of the objects.
	# The score is shown on the result image, together with the class label.
	detection_scores = detection_graph.get_tensor_by_name('detection_scores:0')
	detection_classes = detection_graph.get_tensor_by_name('detection_classes:0')

	# Number of objects detected
	num_detections = detection_graph.get_tensor_by_name('num_detections:0')
	return sess,image_tensor, detection_boxes, detection_scores, detection_classes, num_detections


def run(frame,sess,image_tensor,detection_boxes, detection_scores, detection_classes, num_detections):

	frame_expanded = np.expand_dims(frame, axis=0)

    # Perform the actual detection by running the model with the image as input
	(boxes, scores, classes, num) = sess.run(
		[detection_boxes, detection_scores, detection_classes, num_detections],
		feed_dict={image_tensor: frame_expanded})
        
	return boxes, scores, classes, num


if __name__=="__main__":

    root = tk.Tk()
    root.withdraw()

    my_dict=label_map_util.get_label_map_dict(PATH_TO_LABEL)
    inv_dict = {v: k for k, v in my_dict.items()}

    n_classes=len(inv_dict)
    XML_PATH=askdirectory(title="Select XML folder")
    IMAGES_PATH=askdirectory(title="Select Images folder")

    xml_begin='''<annotation>
            <folder>%s</folder>
            <filename>%s</filename>
            <path>%s</path>
            <source>
                    <database>Unknown</database>
            </source>
            <size>
                    <width>%d</width>
                    <height>%d</height>
                    <depth>%d</depth>
            </size>
            <segmented>0</segmented>'''
    xml_face='''\n	<object>
                    <name>%s</name>
                    <pose>Unspecified</pose>
                    <truncated>0</truncated>
                    <difficult>0</difficult>
                    <bndbox>
                            <xmin>%d</xmin>
                            <ymin>%d</ymin>
                            <xmax>%d</xmax>
                            <ymax>%d</ymax>
                    </bndbox>
            </object>'''

    xml_end="\n</annotation>"

    logfile=open("errors.log","w")

    # create the detector, using default weights

    img_list = os.listdir(IMAGES_PATH)

    n_imgs=len(img_list)
    j=0

    #init detector
    sess,image_tensor,detection_boxes, detection_scores, detection_classes, num_detections=load_model(PATH_TO_CKPT, PATH_TO_LABEL, n_classes)

    for i in progressbar.progressbar(range(n_imgs)):

        imgname=img_list[j]
        if "desktop.ini" not in imgname:
            # load image from file
            pixels = pyplot.imread(IMAGES_PATH+'/'+imgname)
            # get image size
            h,w,d=pixels.shape

            #detect
            boxes, scores, classes, num = run(pixels,sess,image_tensor,detection_boxes, detection_scores, detection_classes, num_detections)

            
            imgfolder=IMAGES_PATH.split(os.sep)[-1]
            #imgname=filename.split(os.sep)[-1]
            imgpath=IMAGES_PATH+'/'+imgname
            xmlname=('.').join(imgname.split('.')[:-1])

            # create annotation .xml file
            f=open(XML_PATH+'/'+xmlname+'.xml',"w")

            # write begin of xml file
            text=xml_begin%(imgfolder,imgname,imgpath,w,h,d)
            f.write(text)

            faces=[]
            labels=[]
            for i,score in enumerate(scores[0]):
                if score>MIN_SCORE:
                    boxes[0][i][0]=int(boxes[0][i][0]*pixels.shape[0])
                    boxes[0][i][1]=int(boxes[0][i][1]*pixels.shape[1])
                    boxes[0][i][2]=int(boxes[0][i][2]*pixels.shape[0])
                    boxes[0][i][3]=int(boxes[0][i][3]*pixels.shape[1])
                    faces.append([int(boxes[0][i][1]),int(boxes[0][i][3]),int(boxes[0][i][0]),int(boxes[0][i][2])])
                    labels.append(inv_dict[int(classes[0][i])]) 
            
            # add the detected faces to xml file
            for inde,face in enumerate(faces):
                x_min,x_max,y_min,y_max=face
                text=xml_face%(labels[inde],x_min,y_min,x_max,y_max)
                f.write(text)

            # write end of xml file
            f.write(xml_end)
            f.close()

        j=j+1
            
    logfile.close()
